extern zend_class_entry *php_midgard_query_holder_class;
extern zend_class_entry *php_midgard_query_property_class;
extern zend_class_entry *php_midgard_query_storage_class;
extern zend_class_entry *php_midgard_query_constraint_simple_class;

PHP_MINIT_FUNCTION(midgard2_query_storage);
PHP_MINIT_FUNCTION(midgard2_query_holders);
PHP_MINIT_FUNCTION(midgard2_query_constraints);
PHP_MINIT_FUNCTION(midgard2_query_executors);
